#include <Wire.h>
//=====================================================================
//Rainbowduino Seeedmaster V2
//Originally provided by Seeed Studio
//Modified by Scott C.
//=====================================================================

//=====================================================================
//This is master-side code that contains some prebuilt functions for
//sending commands to a Rainbowduino running V2.7 or later
//The original commands have not been changed, so this code
//could theoretically send commands to V2, but only 'R' commands.  Both
//the master and slave code have been modified to support multiple data
//transmission session over the IIC communications bus, due to the Wire
//Library limitation of only 32 bytes per transmission.  This is most
//evident in changes to the function SendCMD.
//
//In addition a few new command types have been added.  A command 
//starting with 'F' sends color data for each of the 64 leds set up
//similiar to the buffer on the Rainbowduino (byte per two leds).  This
//allows for complex animations to be calculated on the master, and
//then the "frame" of data for all leds to be sent to the slave
//Rainbowduino.  This is particularly helpful for animations involving
//more than one Rainbowduino comprising a "screen."
//
//There are also three other command types dubbed "chromatics."
//Monochromatic (one color), dichromatic (two), and trichromatic 
//(three) frames can be sent with commands starting with 'M','D', and
//'T' respectively.  Each of these commands consists of the relavent
//number of RGB values, followed by 8 bytes per unique color.  Each
//byte in one of these sets represents a line on the display, 1 meaning
//the led is on, 0 off.  These commands are much smaller, and transmit
//in the first transmission session so they are much faster, especially
//important for large screens with many Rainbowduinos.
//
//It is highly recommended for 'F' commands and large screens to modify
//the Wire library for the faster 400KHz transmission.
//Info on how to do this can be found here:
//http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1241668644/0
//
//VERY BIG NOTE: The functions outlined below, specifically ShowFrame,
//can very quickly eat up your available SRAM.  I developed this on the
//older 168 chip, and kept a very keen eye for minimizing SRAM usage
//with unsigned char datatypes whereever possible.  However, a single
//input array of the kind in ShowFrame is 96 bytes, 1/10th of the 1K
//SRAM is gone right there.  the 168 has made way for the 328, but I
//recommend becoming familiar with PROGMEM and rewrite your functions
//to extract data from flash memory, rather than defining the data like
//normal variables.  Not having enough SRAM shows up as very bizarre 
//errors that are difficult to track down and fix, and you will not
//be notified at compile time.  Moving images and data to PROGMEM 
//allows you to see how much storage you are using at compile time, 
//since PROGMEM variables are stored in flash and become part of the 
//sketch size in flash memory.  Overstepping the available flash memory 
//will throw an error when compiled/uploaded.
//
//The screen I rewrote this code for is 6 Rainbowduinos, and the master 
//stores over 30 images using chromatics and a handful of full RGB 
//images and is only 7.5K in flash memory.  Some examples of this can
//be found by searching Youtube for user ThreeFN.
//======================================================================



unsigned char RainbowCMD[5];
unsigned char State = 0;  
unsigned long timeout;


void setup()
{
  Wire.begin(); // join i2c bus (address optional for master) 
}

void loop()
{
  int i;
  for (i=0;i<5;i++)
  {
    ShowImage(4,i,0);
    delay(500);
  }

  int test;
  unsigned char shift;
  for(test='0';test<'9';test++)
  {
    for(shift=0;shift<16;shift++){
      ShowChar(4,test,15,0,0,shift);
      delay(100);
    }
  }

  for(test='A';test<'Z';test++)
  {
    for(shift=0;shift<16;shift++){
      ShowChar(4,test,0,15,0,shift);
      delay(500);
    }
  }

  for(test='a';test<'z';test++)
  {
    for(shift=0;shift<16;shift++){
      ShowChar(4,test,0,0,15,shift);
      delay(100);
    }
  }

}


//--------------------------------------------------------------------------
//Name:ShowColor
//function: Send a conmand to Rainbowduino for showing a color
//parameter: Address: rainbowduino IIC address
//                 red,green,blue:  the color RGB    
//----------------------------------------------------------------------------
void ShowColor(int Address,unsigned char red , unsigned char green, unsigned char blue)
{
  RainbowCMD[0]='R';
  RainbowCMD[1]=0x03;
  RainbowCMD[2]=red;
  RainbowCMD[3]=((green<<4)|(blue));

  SendCMD(Address);
}


//--------------------------------------------------------------------------
//Name:ShowImage
//function: Send a conmand to Rainbowduino for showing a picture which was pre-set in Rainbowduino Flash
//parameter: Address: rainbowduino IIC address
//                 number:  the pre-set picture position
//                 shift: the picture  shift bit for display
//----------------------------------------------------------------------------
void ShowImage(int Address, unsigned char number,unsigned char shift)
{
  RainbowCMD[0]='R';
  RainbowCMD[1]=0x01;
  RainbowCMD[2]=(shift<<4);
  RainbowCMD[4]=number;

  SendCMD(Address);
}


//--------------------------------------------------------------------------
//Name:ShowChar
//function: Send a conmand to Rainbowduino for showing a color
//parameter: Address: rainbowduino IIC address
//                 red,green,blue:  the color RGB  
//                 shift: the picture  shift bit for display
//                 ASCII:the char or Number want to show
//----------------------------------------------------------------------------
void ShowChar(int  Address,unsigned char ASCII,unsigned char red, unsigned char blue ,unsigned char green,unsigned char shift)
{
  RainbowCMD[0]='R';
  RainbowCMD[1]=0x02;
  RainbowCMD[2]=((shift<<4)|(red));
  RainbowCMD[3]=((green<<4)|(blue));
  RainbowCMD[4]=ASCII;

  SendCMD(Address);
}


//---------------------------------------------------------------------------
//Name:ShowFrame
//function: Send a frame to rainbowduino for displaying
//parameter: Address: rainbowduino IIC address
//           Frame: The array to be sent formatted as [color][row][dots]  
//           *Note: The 4 bit value for two leds brightnesses is stored as a 
//                  byte, so each dots is two leds side by side
//           *Eg: dots = 0xF3 would be first led at max brightness and second 
//                at 3.
//----------------------------------------------------------------------------
void ShowFrame(unsigned char  Address, unsigned char Frame[3][8][4])
{
  unsigned char color,row,dots;

  RainbowCMD[0][0]='F';
  for (color=0;color<3;color++){
    for (row=0;row<8;row++){
      for (dots=0;dots<4;dots++){
        RainbowCMD[color+1][(4*row)+dots]=Frame[color][row][dots];
      }
    }
  }

  SendCMD(Address);
}
//---------------------------------------------------------------------------
//Name:ShowMono
//function: Send a monochromatic frame to rainbowduino for displaying
//parameter: Address: rainbowduino IIC address
//           red,green,blue: the color to show when a led is on, 0-15 per color
//           Frame: The array to be sent formatted as [row]
//           *Note: the byte of each row is one bit per led, and where it's 
//                  on or off (1 or 0), cannot be written with B10101010
//----------------------------------------------------------------------------
void ShowMono(unsigned char  Address, unsigned char red, unsigned char green, unsigned char blue, unsigned char Frame[8])
{
  unsigned char row;

  RainbowCMD[0][0]='M';
  RainbowCMD[0][1]=(red&0x0F);
  RainbowCMD[0][2]=((green&0x0F)<<4)|(blue&0x0F);
  for (row=0;row<8;row++){
    RainbowCMD[0][row+3]=Frame[row];
  }

  SendCMD(Address);
}

//---------------------------------------------------------------------------
//Name:ShowDich
//function: Send a dichromatic frame to rainbowduino for displaying
//parameter: Address: rainbowduino IIC address
//           red1,green1,blue1,red2,green2,blue2: the color to show when a led is on
//           for each of two colors, 0-15
//           *Note: a 1 color will overwrite a 2 color occupying the same led
//                  this can be useful for just adding on another layer of color to
//                  an existing image
//           Frame: The array to be sent formatted as [color][row]
//           *Note: the byte of each row is one bit per led, and where it's 
//                  on or off (1 or 0), cannot be written with B10101010
//----------------------------------------------------------------------------
void ShowDich(unsigned char  Address, unsigned char red1, unsigned char green1, unsigned char blue1, unsigned char red2, unsigned char green2, unsigned char blue2, unsigned char Frame[2][8])
{
  unsigned char color,row;

  RainbowCMD[0][0]='D';
  RainbowCMD[0][1]=((red1&0x0F)<<4)|(green1&0x0F);
  RainbowCMD[0][2]=((blue1&0x0F)<<4)|(red2&0x0F);
  RainbowCMD[0][3]=((green2&0x0F)<<4)|(blue2&0x0F);
  
  for (color=0;color<2;color++){
    for (row=0;row<8;row++){
      RainbowCMD[0][(color*8)+row+4]=Frame[color][row];
    }
  }
  SendCMD(Address);
}


//---------------------------------------------------------------------------
//Name:ShowTrich
//function: Send a trichromatic frame to rainbowduino for displaying
//parameter: Address: rainbowduino IIC address
//           red1,green1,blue1,red2,green2,blue2,red3,green3,blue3: the color 
//               to show when a led is on for each of two colors, 0-15
//           *Note: a 1 color will overwrite a 2 color, etc occupying the same led
//                  this can be useful for just adding on another layer of color to
//                  an existing image, 1 > 2 > 3
//           Frame: The array to be sent formatted as [color][row]
//           *Note: the byte of each row is one bit per led, and where it's 
//                  on or off (1 or 0), cannot be written with B10101010
//----------------------------------------------------------------------------
void ShowTrich(unsigned char  Address, unsigned char red1, unsigned char green1, unsigned char blue1, unsigned char red2, unsigned char green2, unsigned char blue2, unsigned char red3, unsigned char green3, unsigned char blue3, unsigned char Frame[3][8])
{
  unsigned char color,row;

  RainbowCMD[0][0]='T';
  RainbowCMD[0][1]=((red1&0x0F)<<4)|(green1&0x0F);
  RainbowCMD[0][2]=((blue1&0x0F)<<4)|(red2&0x0F);
  RainbowCMD[0][3]=((green2&0x0F)<<4)|(blue2&0x0F);
  RainbowCMD[0][4]=(red3&0x0F);
  RainbowCMD[0][5]=((green3&0x0F)<<4)|(blue3&0x0F);
  
  for (color=0;color<3;color++){
    for (row=0;row<8;row++){
      RainbowCMD[0][(color*8)+row+6]=Frame[color][row];
    }
  }
  SendCMD(Address);
}


//--------------------------------------------------------------------------
//Name:SendCMD
//function: Send a 5 byet Rainbow conmand out 
//parameter: NC  
//----------------------------------------------------------------------------
void SendCMD(int  Add)
{   
  unsigned char OK=0;
  unsigned char i,temp;

  while(!OK)
  {                          
    switch (State)
    { 	

    case 0:                          
      Wire.beginTransmission(Add);
      for (i=0;i<5;i++) Wire.send(RainbowCMD[i]);
      Wire.endTransmission();    
      delay(5);   
      State=1;                      
      break;

    case 1:

      Wire.requestFrom(Add,1);   
      if (Wire.available()>0) 
        temp=Wire.receive();    
      else {
        temp =0xFF;
        timeout++;
      }

      if ((temp==1)||(temp==2)) State=2;
      else if (temp==0) State=0;

      if (timeout>5000)
      {
        timeout=0;
        State=0;
      }

      delay(5);
      break;

    case 2:
      OK=1;
      State=0;
      break;

    default:
      State=0;
      break;

    }
  }
}
